/* Copyright (C) 2000-2015 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#ifndef _UDM_PROG_H
#define _UDM_PROG_H

#include "udm_lex.h"

#define UDM_PROG_MAXARG 5

typedef enum udm_tmpl_cmd
{
  UDM_PROG_NOP= 0,
  UDM_PROG_JE,
  UDM_PROG_JNE,
  UDM_PROG_JMP,
  /* Unary integer operators */
  UDM_PROG_NOT_IREG0,
  UDM_PROG_INC_IREG0,
  UDM_PROG_DEC_IREG0,
  UDM_PROG_INC_DREG0,
  UDM_PROG_DEC_DREG0,
  UDM_PROG_INC_CHR_PTR_STACK_OFFSET,
  UDM_PROG_DEC_CHR_PTR_STACK_OFFSET,
  UDM_PROG_INC_INT_PTR_STACK_OFFSET,
  UDM_PROG_DEC_INT_PTR_STACK_OFFSET,
  UDM_PROG_INC_DBL_PTR_STACK_OFFSET,
  UDM_PROG_DEC_DBL_PTR_STACK_OFFSET,
  /* Dynadic integer arithmetic and bit operators */
  UDM_PROG_MUL_IREG0_IREG1,
  UDM_PROG_ADD_IREG0_IREG1,
  UDM_PROG_SUB_IREG0_IREG1,
  UDM_PROG_AND_IREG0_IREG1,
  UDM_PROG_OR_IREG0_IREG1,
  UDM_PROG_XOR_IREG0_IREG1,
  UDM_PROG_SHL_IREG0_IREG1,
  UDM_PROG_SHR_IREG0_IREG1,
  UDM_PROG_DIV_IREG0_IREG1,
  UDM_PROG_REM_IREG0_IREG1,
  /* Int comparison operators */
  UDM_PROG_CMP_IREG0_IREG1_EQ,
  UDM_PROG_CMP_IREG0_IREG1_LT,
  UDM_PROG_CMP_IREG0_IREG1_GT,
  UDM_PROG_CMP_IREG0_IREG1_LE,
  UDM_PROG_CMP_IREG0_IREG1_GE,
  UDM_PROG_CMP_IREG0_IREG1_NE,

  
  /* Unary double operators */
  UDM_PROG_MOV_IREG0_0,
  UDM_PROG_MOV_DREG0_0,
  UDM_PROG_MOV_IREG1_0,
  UDM_PROG_MOV_DREG1_0,
  UDM_PROG_MOV_HREG0_0,
  UDM_PROG_MOV_SREG0_0,
  UDM_PROG_MOV_IREG0_INT_PTR_STACK_OFFSET,
  UDM_PROG_MOV_DREG0_DBL_PTR_STACK_OFFSET,
  UDM_PROG_MOV_IREG1_INT_PTR_STACK_OFFSET,
  UDM_PROG_MOV_DREG1_DBL_PTR_STACK_OFFSET,
  UDM_PROG_MOV_SREG0_STACK_OFFSET,
  UDM_PROG_MOV_VREG0_STACK_OFFSET,
  UDM_PROG_MOV_VREG1_STACK_OFFSET,
  UDM_PROG_MOV_VREG2_STACK_OFFSET,
  UDM_PROG_MOV_VREG3_STACK_OFFSET,
  UDM_PROG_MOV_VREG4_STACK_OFFSET,

  /* Copy operators with constants */
  UDM_PROG_MOV_IREG0_INT,
  UDM_PROG_MOV_IREG1_INT,
  UDM_PROG_MOV_DREG0_DBL,
  UDM_PROG_MOV_DREG1_DBL,
  UDM_PROG_MOV_HREG0_PTR,

  /* Double arithmetic operators */
  UDM_PROG_MUL_DREG0_DREG1,
  UDM_PROG_ADD_DREG0_DREG1,
  UDM_PROG_SUB_DREG0_DREG1,
  UDM_PROG_DIV_DREG0_DREG1,
  
  /* Double comparison operators */
  UDM_PROG_CMP_DREG0_DREG1_EQ,
  UDM_PROG_CMP_DREG0_DREG1_LT,
  UDM_PROG_CMP_DREG0_DREG1_GT,
  UDM_PROG_CMP_DREG0_DREG1_LE,
  UDM_PROG_CMP_DREG0_DREG1_GE,
  UDM_PROG_CMP_DREG0_DREG1_NE,

  /* Dyadic copy operators */
  UDM_PROG_MOV_DREG1_DREG0,
  UDM_PROG_MOV_IREG1_IREG0,
  UDM_PROG_MOV_INT_PTR_S0_IREG0,
  UDM_PROG_MOV_DBL_PTR_S0_DREG0,
  UDM_PROG_MOV_CHR_PTR_S0_IREG0,

  /* Dyaric copy with conversion */
  UDM_PROG_MOV_DREG0_IREG0,
  UDM_PROG_MOV_DREG1_IREG0,
  UDM_PROG_MOV_IREG0_DREG0,
  UDM_PROG_MOV_IREG1_DREG1,
  UDM_PROG_MOV_DREG1_IREG1,

  /* Exchange operators */
  UDM_PROG_SWP_DREG0_DREG1,
  UDM_PROG_SWP_IREG0_IREG1,


  /* Built-in function and command calls */
  UDM_PROG_CALL,
  UDM_PROG_HANDLED,
  UDM_PROG_EXIT
} udm_tmpl_cmd_t;


typedef struct udm_tmpl_names_st
{
  const UDM_CONST_STR name;
  size_t nargs;
  udm_func_runtime_t exec;
} UDM_PROG_CMD;


typedef struct
{
  size_t offset;
  size_t length;
} udm_offset_and_length_t;


typedef struct
{
  const UDM_VALUE_HANDLER *handler;
  size_t stack_offset;
} UDM_PROG_VAR_VALUE;


typedef struct
{
  UDM_CONST_TOKEN name;
  UDM_PROG_VAR_VALUE value;
} UDM_PROG_VAR;


typedef struct
{
  char *name;
  UDM_PROG_VAR_VALUE value;
} UDM_PROG_VAR2;


typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_PROG_VAR2 *Item;
  size_t next_stack_offset;
} UDM_PROG_VARLIST;


typedef struct
{
  size_t nitems;
  size_t mitems;
  UDM_PROG_VARLIST *Item;
} UDM_PROG_VARLISTLIST;


void UdmProgVarListInit(UDM_PROG_VARLIST *List);
void UdmProgVarListFree(UDM_PROG_VARLIST *List);
udm_rc_t UdmProgVarListAdd(UDM_PROG_VARLIST *List,
                           const char *name,
                           const UDM_PROG_VAR_VALUE *Value);
const UDM_PROG_VAR2 *UdmProgVarListFind(const UDM_PROG_VARLIST *List,
                                        const char *name);

void UdmProgVarListListInit(UDM_PROG_VARLISTLIST *List);
void UdmProgVarListListFree(UDM_PROG_VARLISTLIST *List);
udm_rc_t UdmProgVarListListAdd(UDM_PROG_VARLISTLIST *List, const UDM_PROG_VARLIST *Item);


typedef struct
{
  size_t stack_offset;
  udm_offset_and_length_t textdata_addr;
  const UDM_VALUE_HANDLER *handler;
} UDM_PROG_ARG;


typedef struct udm_prog_item_st
{
  udm_tmpl_cmd_t cmdnum;
  char *comment;
  UDM_PROG_ARG item_args;
  union
  {
    size_t jump;
    int val_int;
    double val_double;
    const void *val_const_ptr;
    size_t offset;
    const UDM_FUNCTION *func;
  } cmdarg;
  UDM_PROG_CMD *cmd;
} UDM_PROG_ITEM;


typedef struct udm_tmpl_prg_st
{
  size_t nitems;
  size_t mitems;
  UDM_PROG_ITEM *Items;
  UDM_DSTR textdata;
} UDM_PROG;


typedef struct udm_prog_executor_arg_st
{
  char *Data;
} UDM_PROG_EXECUTOR_ARG;


typedef struct udm_prog_global_variables_st
{
  int argc;
  char **argv;
  FILE *stdout;
  UDM_THDHANDLER THDHandler;
} UDM_PROG_GLOBAL_VARIABLES;


typedef struct udm_prog_execotor_registers_st
{
  int     i[2];
  double  d[2];
  size_t  z0;
  const char *cs0;
  const UDM_VALUE_HANDLER *h0;
  char *s0;                    /* e.g. values returned from functions */
} UDM_PROG_EXECUTOR_REGISTERS;


typedef struct udm_prog_executor_state_st
{
  const UDM_PROG_ITEM *curr;
  const UDM_PROG *prog;
  UDM_PROG_EXECUTOR_REGISTERS reg;  
  UDM_DSTR stack;
  UDM_PROG_EXECUTOR_ARG vreg[UDM_PROG_MAXARG];
  UDM_PROG_GLOBAL_VARIABLES globals;
  udm_bool_t fatal_error;
} UDM_PROG_EXECUTOR_STATE;

#define UDM_FUNC_RETURN_INT(st,x)     ((st)->reg.i[0]= (x))
#define UDM_FUNC_RETURN_DOUBLE(st,x)  ((st)->reg.d[0]= (x))

typedef struct udm_prog_executor_st
{
  UDM_PROG_EXECUTOR_STATE state;
} UDM_PROG_EXECUTOR;


typedef enum
{
  UDM_PROG_STMT_IF,
  UDM_PROG_STMT_FOR,
  UDM_PROG_STMT_WHILE
} udm_tmpl_block_stmt_t;


typedef struct udm_tmpl_compile_stack_item_st
{
  size_t beg;
  size_t jmp;
  udm_tmpl_block_stmt_t stmt; /* IF FOR WHILE */
} UDM_PROG_COMPILE_STACK_ITEM;


typedef struct udm_tmpl_compile_stack_st
{
  size_t nitems;
  size_t mitems;
  UDM_PROG_COMPILE_STACK_ITEM *Items;
} UDM_PROG_COMPILE_STACK;


typedef enum 
{
  COMMENT_NONE= 0,
  COMMENT_INLINE= 1,
  COMMENT_BLOCK= 2
} udm_template_comment_type_t;


typedef struct udm_template_helper_st
{
  UDM_LEX_SCANNER scanner;
  UDM_PROG *prg;
  UDM_PROG_COMPILE_STACK st;
  UDM_PROG_VARLISTLIST Vars2;
  udm_template_comment_type_t comment_type;
  size_t lineno;
  const char *start;  /* The beginning of the current template section */
  char errstr[128];
  size_t pi_count;
  size_t current_loop_continue;
  udm_bool_t generate_debug_info;
} UDM_PROG_COMPILER;


void UdmProgInit(UDM_PROG *p);
void UdmProgPrint(const UDM_PROG *p);
void UdmProgFree(UDM_PROG *prg);


void UdmProgCompilerInit(UDM_PROG_COMPILER *compiler,
                         size_t lineno, UDM_PROG *prg,
                         const char *src, size_t srclen);
void UdmProgCompilerFree(UDM_PROG_COMPILER *compiler);

void UdmProgExecutorInit(UDM_PROG_EXECUTOR *executor,
                         FILE *stream, UDM_DSTR *dstr);
int  UdmProgExecutorExec(UDM_PROG_EXECUTOR *tmpl, const UDM_PROG *prg);
void UdmProgExecutorFree(UDM_PROG_EXECUTOR *executor);

udm_rc_t UdmCompilePI(UDM_PROG_COMPILER *compiler, const UDM_CONST_STR *src);
udm_rc_t UdmCompilePIProgram(UDM_PROG_COMPILER *compiler, const UDM_CONST_STR *src);

/******************************/
UDM_PROG *UdmProgCompilerGetProg(UDM_PROG_COMPILER *tmpl);

void  UdmProgItemInit(UDM_PROG_ITEM *i);
udm_rc_t UdmProgAdd(UDM_PROG *prg, UDM_PROG_ITEM *item);
udm_rc_t UdmProgAddArg0Simple(UDM_PROG *prog, UDM_PROG_CMD *cmd);
udm_rc_t UdmProgAddArg0SimpleOp(UDM_PROG *prog, udm_tmpl_cmd_t op);
udm_rc_t UdmProgAddArg1SimpleOpDouble(UDM_PROG *pro, udm_tmpl_cmd_t op, double val);
udm_rc_t UdmProgAddArg1SimpleOpInt(UDM_PROG *pro, udm_tmpl_cmd_t op, int val);
udm_rc_t UdmProgAddJmp(UDM_PROG *prog, udm_tmpl_cmd_t cmdnum, int addr);
void UdmProgFixJumpInRange(UDM_PROG *prg, size_t start, size_t end);
udm_rc_t UdmProgAddValueHandler(UDM_PROG *prog, udm_tmpl_cmd_t cmdnum,
                                const UDM_VALUE_HANDLER *handler);
size_t PrintTextTemplate(UDM_ENV *Env, FILE *stream,
                         UDM_DSTR *dstr, UDM_VARLIST *vars,
                         const char *templ,
                         const char *HlBeg, const char *HlEnd,
                         udm_bool_t auto_escape);
size_t out_string(FILE *stream, UDM_DSTR *dstr, const char * src);

void UdmProgExecutorStateInitArgs(UDM_PROG_EXECUTOR_STATE *state);
void UdmProgExecutorStateReturnStrn(UDM_PROG_EXECUTOR_STATE *state,
                                    const char *str, size_t length);
void *UdmProgExecutorStateSetupReturnValue(UDM_PROG_EXECUTOR_STATE *state,
                                           const UDM_VALUE_HANDLER *handler);

extern const UDM_FUNCTION udm_builtin_functions[];
void UdmFunction_exit(UDM_PROG_EXECUTOR_STATE *state);

#endif
