/* Copyright (C) 2000-2015 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#ifndef _UDM_SEARCH_TOOL_H
#define _UDM_SEARCH_TOOL_H

#define UDM_FAST_PRESORT_DOCS	300
#define UDM_DEFAULT_USER_WORD_WEIGHT 256

typedef unsigned long long udm_found_word_bitmask_t;


typedef struct
{
  udm_found_word_bitmask_t mask;
  uint4 quality;
} UDM_SNIPPETCHUNK_STAT;

void UdmSnippetChunkStatInit(UDM_SNIPPETCHUNK_STAT *);

typedef struct
{
  UDM_CONV src_uni;
  UDM_CONV uni_dst;
  UDM_CONV uni_wcs;
  int hlstop;
  int segmenter;
} UDM_HIGHLIGHT_CONV;


typedef struct udm_search_query_param_st
{
  double NumWordFactor;
  unsigned int NumSections;
  unsigned int MinCoordFactor;
  unsigned int MaxCoordFactor;
  unsigned int Phrase2CountFactor;
  unsigned int Phrase3CountFactor;
  unsigned int WordFormFactor;
  unsigned int WordDensityFactor;
  unsigned int SkipWordDistanceThreshold;
  unsigned int WordDistanceWeight;
  unsigned int IDFFactor;
  urlid_t DebugURLId;
  unsigned int SingleWordDistance;
  unsigned int NumDistinctWordFactor;
  int UserScoreFactor;
  int PopularityFactor;
  int RelevancyFactor;
  int DateFactor;
  udm_bool_t SaveSectionSize;
  udm_bool_t NewVersion;
  char wf[256];
  char nwf[256];
  unsigned int nwf_num;
} UDM_QUERY_PARAM;

void UdmQueryParamInit(UDM_QUERY_PARAM *prm, UDM_ENV *Env, UDM_VARLIST *dbvars);


/* Functions form urldata.c */
void UdmURLDataFree(UDM_URLDATA *D);
void UdmURLDataListInit(UDM_URLDATALIST *L);
void UdmURLDataSortBySite(UDM_URLDATALIST *L);
void UdmURLDataSortByPattern(UDM_URLDATALIST *D, const char *pattern);
void UdmURLDataListSort(UDM_URLDATALIST *L);
UDM_URLDATA *UdmURLDataListSearch(UDM_URLDATALIST *List, urlid_t id);
void UdmURLDataListFreeItems(UDM_URLDATALIST *List, size_t first, size_t last);
void UdmURLDataListFree(UDM_URLDATALIST *List);
void UdmURLDataListClearParams(UDM_URLDATALIST *List);
size_t UdmURLDataCompact(UDM_URLDATA *dst, UDM_URLDATA *src, size_t n);

/* Functions from groupby.c */
void UdmURLDataGroupBySite(UDM_URLDATALIST *L);
udm_rc_t UdmURLDataListGroupBySiteUsingSort(UDM_AGENT *A,
                                            UDM_URLDATALIST *R,
                                            UDM_DB *db);
void UdmURLDataApplySiteRank(UDM_AGENT *A, UDM_URLDATALIST *DataList,
                             int is_aggregation_point);
void UdmURLDataListApplyPopularity(UDM_AGENT *A,
                                   UDM_URLDATALIST *URLData,
                                   const UDM_QUERY_PARAM *prm);

udm_rc_t UdmURLDataListPackSite(UDM_URLDATALIST *List, UDM_DSTR *dstr);
udm_rc_t UdmURLDataListUnpackSite(UDM_AGENT *A, UDM_URLDATALIST *List,
                                  const UDM_CONST_STR *str_arg);

/* Functions from distance.c */
typedef struct udm_distance_stat_st
{
  uint4 sum;
  uint4 num;
  uint4 phrase2_count;
  uint4 phrase3_count;
} UDM_WORD_DISTANCE_STAT;

void UdmWordDistanceStatInit(UDM_WORD_DISTANCE_STAT *d);

void CalcAverageWordDistance(size_t wf2_secno,
                             UDM_COORD2 *phr, size_t num, size_t nuniq,
                             UDM_WORD_DISTANCE_STAT *dist);
                                                 
/* Functions from score.c */
udm_rc_t UdmUserScoreListApplyToURLScoreList(UDM_AGENT *A,
                                             UDM_URLSCORELIST *URLScoreList,
                                             UDM_URL_INT4_LIST *UserScoreList,
                                             const UDM_QUERY_PARAM *prm);
udm_rc_t UdmUserScoreListApplyToURLDataList(UDM_AGENT *A,
                                            UDM_URLDATALIST *URLDataList,
                                            UDM_URL_INT4_LIST *UserScoreList,
                                            const UDM_QUERY_PARAM *prm);

udm_rc_t
UdmURLDataListApplyRelevancyFactors(UDM_AGENT *Agent,
                                    UDM_URLDATALIST *DataList,
                                    const UDM_QUERY_PARAM *prm);

void UdmURLScoreListSortByScore(UDM_URLSCORELIST *ScoreList);
void UdmURLScoreListSortByScoreThenURLTop(UDM_URLSCORELIST *ScoreList,
                                                 size_t topcount);
void UdmURLScoreListSortByScoreThenURL(UDM_URLSCORELIST *ScoreList);

void UdmGroupByURL2(UDM_AGENT *Agent, UDM_DB *db, UDM_QUERY *Query,
                    const UDM_QUERY_PARAM *query_param,
                    UDM_SEARCHSECTIONLIST *CoordList,
                    UDM_URLSCORELIST *ScoreList);

udm_rc_t UdmQueryPrepare(UDM_AGENT *query, UDM_QUERY *Query);
udm_rc_t UdmApplyFastLimit(UDM_URLCRDLIST *Coord, UDM_URLID_LIST *urls);

/* Functions from wordinfo.c */
udm_rc_t UdmQueryWordInfo(UDM_QUERY *Query);


/* Functions from fuzzy.c */
udm_rc_t UdmAllForms(UDM_AGENT *Indexer, UDM_WIDEWORDLIST *result,
                     UDM_WIDEWORD *uword);

udm_rc_t UdmComplexSynonyms(UDM_AGENT *Indexer, UDM_WIDEWORDLIST *result);

/* Functions from highlight.c */
void UdmExcerptConvInit(UDM_HIGHLIGHT_CONV *cnv,
                        UDM_CHARSET *wcs,
                        UDM_CHARSET *src,
                        UDM_CHARSET *dst,
                        int hlstop, int segmenter);
void UdmExcerptConvInitFromEnv(UDM_HIGHLIGHT_CONV *cnv,
                               UDM_CHARSET *wcs,
                               UDM_CHARSET *src,
                               UDM_CHARSET *dst,
                               UDM_ENV *Env);
size_t
UdmHlConvertExtWithConv(UDM_AGENT *A,
                        UDM_DSTR *dstr,
                        UDM_SNIPPETCHUNK_STAT *stat,
                        UDM_WIDEWORDLIST *List,
                        const char *src, size_t srclen,
                        UDM_HIGHLIGHT_CONV *ec);
udm_rc_t
UdmVarListHlConvert(UDM_AGENT *A,
                    UDM_WIDEWORDLIST *WWList,
                    UDM_VARLIST *Vars,
                    UDM_HIGHLIGHT_CONV *ec);

udm_rc_t UdmQueryConvert(UDM_AGENT *A, UDM_QUERY *Query,
                         UDM_CHARSET *lcs, UDM_CHARSET *bcs);
size_t UdmRemoveHiLight(char *dst, size_t dstlength,
                        const char *src, size_t srclength);
size_t UdmRemoveHl(UDM_CHARSET *cs, char *str, size_t from, size_t to);
char* UdmRemoveHiLightDup(const char *s);

/* Functions from segment.c */
int *UdmUniSegment(UDM_AGENT *Indexer, int *s, const char *lang, const char *seg);
int *UdmUniSegmentByType(UDM_AGENT *Indexer, int *s, int type, int ch);
int UdmUniSegmenterFind(UDM_ENV *Env, const char *lang, const char *seg);
udm_rc_t UdmTextListSegment(UDM_AGENT *Indexer, UDM_TEXTLIST *tlist,
                            UDM_CHARSET *cs, int type);

/* Functions from section.c */
UDM_SEARCHSECTION *UdmSearchSectionListFind(UDM_SEARCHSECTIONLIST *SectionList,
                                            urlid_t url_id);
void UdmSearchSectionListPrint(UDM_SEARCHSECTIONLIST *SectionList);
udm_rc_t UdmSearchSectionListAlloc(UDM_SEARCHSECTIONLIST *List, size_t ncoords, size_t nsections);
void UdmSearchSectionListFree(UDM_SEARCHSECTIONLIST *List);
udm_rc_t UdmSearchSectionListListAdd(UDM_SEARCHSECTIONLISTLIST *List,
                                     UDM_SEARCHSECTIONLIST *Item);
void UdmSearchSectionListListInit(UDM_SEARCHSECTIONLISTLIST *List);
void UdmSearchSectionListListFree(UDM_SEARCHSECTIONLISTLIST *List);
udm_rc_t UdmSearchSectionListListMergeSorted(UDM_SEARCHSECTIONLISTLIST *SrcList,
                                             UDM_SEARCHSECTIONLIST *Dst, int opt);

/* Functions from urlidlist.c */
int UdmCmpURLID(urlid_t *s1, urlid_t *s2); /* for qsort */
udm_rc_t UdmURLIdListJoin(UDM_URLID_LIST *urls, UDM_URLID_LIST *fl_urls);
udm_rc_t UdmURLIdListUnion(UDM_URLID_LIST *a, UDM_URLID_LIST *b);
udm_rc_t UdmURLIdListCopy(UDM_URLID_LIST *a, UDM_URLID_LIST *b);
udm_rc_t UdmURLIdListMerge(UDM_URLID_LIST *a, UDM_URLID_LIST *b);
void UdmURLIdListSort(UDM_URLID_LIST *a);

/* Functions from popularity.c */
udm_rc_t UdmURLDataListUnpackPopularity(UDM_AGENT *A, UDM_URLDATALIST *URLDataList,
                                        UDM_CONST_STR *cstr);
udm_rc_t UdmURLDataListPackPopularity(UDM_AGENT *A, UDM_URLDATALIST *List,
                                      UDM_DSTR *pop);

#endif
