--
-- Use:
--     db2 -td";" -vf create.sql 
--
-- Using 20 as identity cache, for better performance
--
CREATE TABLE url (
  rec_id          INT          NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 1,INCREMENT BY 1, CACHE 20),
  status          INT          NOT NULL WITH DEFAULT  0,
  docsize         INT          NOT NULL WITH DEFAULT  0,
  next_index_time INT          NOT NULL WITH DEFAULT  0,
  last_mod_time   INT          NOT NULL WITH DEFAULT  0,
  referrer        INT          NOT NULL WITH DEFAULT  0,
  hops            INT          NOT NULL WITH DEFAULT  0,
  crc32           INT          NOT NULL WITH DEFAULT -1,
  seed            INT          NOT NULL WITH DEFAULT  0,
  bad_since_time  INT          NOT NULL WITH DEFAULT  0,
  server_id       INT          NOT NULL WITH DEFAULT  0,
  shows           INT          NOT NULL WITH DEFAULT  0,
  url             VARCHAR(255) NOT NULL WITH DEFAULT '',
  PRIMARY KEY (REC_ID)
) DATA CAPTURE NONE;

CREATE UNIQUE INDEX url_url      ON url (url);
CREATE INDEX url_crc             ON url (crc32);
CREATE INDEX url_seed            ON url (seed);
CREATE INDEX url_referrer        ON url (referrer);
CREATE INDEX url_next_index_time ON url (next_index_time);
CREATE INDEX url_bad_since_time  ON url (bad_since_time);
CREATE INDEX url_server_id       ON url (server_id);
CREATE INDEX url_status          ON url (status);
CREATE INDEX url_hops            ON url (hops);


CREATE TABLE urlinfo (
  url_id INT         NOT NULL,
  sname  VARCHAR(96) NOT NULL,
  sval   CLOB        NOT NULL
);
CREATE INDEX urlinfo_id ON urlinfo (url_id);


CREATE TABLE cachedcopy (
  url_id  INT        NOT NULL,
  ts      INT        NOT NULL,
  content BLOB       NOT NULL
);
CREATE INDEX cachedcopy_id ON cachedcopy (url_id);
CREATE INDEX cachedcopy_ts ON cachedcopy (ts);

CREATE TABLE dict (
  word     VARCHAR(32) NOT NULL,
  url_id   INT NOT NULL,
  coord    INT NOT NULL
) DATA CAPTURE NONE;
CREATE INDEX dict_word    ON dict (word);
CREATE INDEX dict_url_id  ON dict (url_id);


CREATE TABLE bdict (
  word   VARCHAR(255) NOT NULL,
  secno  INT          NOT NULL,
  coords BLOB(128M)   NOT NULL
);
CREATE INDEX bdict_word ON bdict (word);


CREATE TABLE server (
  rec_id     int           not null primary key,
  enabled    int           not null default 0,
  url        VARCHAR(2000) not null default '',
  tag        VARCHAR(96)   not null default '',
  command    char(1)       not null default 'S',
  ordre      int           not null default 0,
  parent     int           not null default 0,
  weight     float         not null default 1,
  pop_weight float         not null default 0
);
CREATE INDEX srv_ordre   ON server (ordre);
CREATE INDEX srv_parent  ON server (parent);
CREATE INDEX srv_command ON server (command);
CREATE INDEX srv_tag     ON server (tag);


CREATE TABLE srvinfo (
  srv_id INT            NOT NULL,
  sname  VARCHAR(96)    NOT NULL,
  sval   VARCHAR(2000) NOT NULL
);
CREATE INDEX srvinfo_id ON srvinfo (srv_id);


CREATE TABLE links (
  url_id   int           not null default 0,
  seed     int           not null default 0,
  url      varchar(1024) not null default '',
  src      varchar(10)   not null default '',
  rel      varchar(32)   not null default '',
  linktext varchar(1024) not null default ''
);
CREATE INDEX key_links_url_id ON links (url_id);
CREATE INDEX key_links_seed ON links (seed);

CREATE TABLE redirect (
  url_id int           not null default 0,
  seed   int           not null default 0,
  url    varchar(1024) not null default ''
);
CREATE INDEX key_redirect_url_id ON redirect (url_id);
CREATE INDEX key_redirect_seed   ON redirect (seed);


CREATE TABLE qtrack (
  rec_id INT            NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 1,INCREMENT BY 1, NO CACHE), 
  ip     VARCHAR(16)    NOT NULL,
  qwords VARCHAR(2000)  NOT NULL,
  qtime  INT            NOT NULL,
  wtime  INT            NOT NULL,
  nfound INT            NOT NULL
);
CREATE INDEX qtrack_ipt ON qtrack(ip,qtime);


CREATE TABLE qinfo (
  q_id  INT NOT NULL,
  sname VARCHAR(64),
  sval  VARCHAR(2000)
);
CREATE INDEX qinfo_id ON qinfo (q_id);

CREATE TABLE wrdstat (
  word varchar(64) NOT NULL,
  snd varchar(64) NOT NULL,
  cnt int NOT NULL
);
CREATE INDEX wrdstat_word ON wrdstat (word);
CREATE INDEX wrdstat_snd  ON wrdstat (snd);

CREATE TABLE qcache (
  id       INT NOT NULL,
  tm       INT NOT NULL,
  doclist  BLOB(128M) NOT NULL,
  wordinfo CLOB(128M) NOT NULL,
  PRIMARY KEY (id,tm)
);
CREATE INDEX key_qcache_tmstamp ON qcache(tm);
