set quoted_identifier on;

--
-- In Sybase we can't use TEXT type like in PgSQL or MySQL:
--
-- The text, ntext, and image data types cannot be compared or sorted, 
-- except when using IS NULL or LIKE operator.
--
-- We use VARCHAR() instead.
--

CREATE TABLE "dict" (
	"url_id" "int" NOT NULL ,
	"word"    varchar (32) NOT NULL ,
	"coord" "int" NOT NULL 
);
CREATE  INDEX "dict_url_id" ON "dict" ("url_id");
CREATE  INDEX "dict_word_url_id" ON "dict" ("word", "url_id");

CREATE TABLE bdict (
  word  VARCHAR(255) NOT NULL,
  secno INT NOT NULL,
  coords IMAGE NOT NULL
);
CREATE INDEX bdict_word ON bdict (word);


CREATE TABLE qcache (
  id       INT NOT NULL,
  tm       INT NOT NULL,
  doclist  IMAGE NOT NULL,
  wordinfo TEXT  NOT NULL,
  PRIMARY KEY (id,tm)
);
CREATE INDEX key_qcache_tmstamp ON qcache(tm);

CREATE TABLE "url" (
        "rec_id"          "int" IDENTITY NOT NULL PRIMARY KEY,
        "status"          "int" DEFAULT 0 NOT NULL,
        "docsize"         "int" DEFAULT 0 NOT NULL,
        "next_index_time" "int" DEFAULT 0 NOT NULL,
        "last_mod_time"   "int" DEFAULT 0 NOT NULL,
        "referrer"        "int" DEFAULT 0 NOT NULL,
        "hops"            "int" DEFAULT 0 NOT NULL,
        "crc32"           "int" DEFAULT -1 NOT NULL,
        "seed"            "int" DEFAULT 0 NOT NULL,
        "bad_since_time"  "int" NULL,
        "server_id"       "int" NULL,
        "shows"           "int" DEFAULT 0 NOT NULL,
        "url"             varchar(600)  NOT NULL,
        CONSTRAINT "UQ_url_url" UNIQUE ("url")
);


CREATE UNIQUE INDEX "url_url" ON "url" ( "url" );
CREATE INDEX "url_crc" ON "url" ( "crc32" );
CREATE INDEX "url_seed" ON "url" ( "seed" );
CREATE INDEX "url_referrer" ON "url" ( "referrer" );
CREATE INDEX "url_next_index_time" ON "url" ( "next_index_time" );
CREATE INDEX "url_status" ON "url" ( "status" );
CREATE INDEX "url_bad_since_time" ON "url" ( "bad_since_time" );
CREATE INDEX "url_hops" ON "url" ( "hops" );
CREATE INDEX "url_server_id" ON url ( "server_id" );


CREATE TABLE "urlinfo" (
       "url_id" "int"         NOT NULL,
       "sname"  VARCHAR(96)   NOT NULL,
       "sval"   TEXT          NOT NULL
);
CREATE INDEX "urlinfo_id" ON "urlinfo" (url_id);

CREATE TABLE "cachedcopy" (
       url_id  int NOT NULL,
       ts      int NOT NULL,
       content IMAGE NOT NULL
);
CREATE INDEX cachedcopy_id ON cachedcopy (url_id);
CREATE INDEX cachedcopy_ts ON cachedcopy (ts);

CREATE TABLE "server" (
        "rec_id"     int           not null primary key,
        "enabled"    int           default 0 not null ,
        "url"        varchar(4096) default '' not null ,
        "tag"        varchar(96)   default '' not null ,
        "command"    char(1)       default 'S' not null ,
        "ordre"      int           default 0 not null ,
        "parent"     int           default 0 not null ,
        "weight"     float         default 1 not null ,
        "pop_weight" float         default 0 not null 
);
CREATE INDEX "srv_ordre"    ON "server" ("ordre");
CREATE INDEX "srv_parent"   ON "server" ("parent");
CREATE INDEX "srv_command"  ON "server" ("command");
CREATE INDEX "srv_tag"      ON "server" ("tag");

CREATE TABLE "srvinfo" (
       srv_id int           NOT NULL,
       sname  varchar(96)   NOT NULL,
       sval   varchar(4096) NOT NULL
);
CREATE INDEX srvinfo_id ON srvinfo (srv_id);


CREATE TABLE links (
  url_id int             default 0  not null,
  seed int               default 0  not null,
  url varchar(2048)      default '' not null,
  src varchar(10)        default '' not null,
  rel varchar(32)        default '' not null,
  linktext varchar(1024) default '' not null
);
CREATE INDEX key_links_url_id ON links (url_id);
CREATE INDEX key_links_seed ON links (seed);


CREATE TABLE redirect (
  url_id int        default 0 not null,
  seed int          default 0 not null,
  url varchar(2048) default '' not null
);
CREATE INDEX redirect_links_url_id ON redirect (url_id);
CREATE INDEX redirect_links_seed   ON redirect (seed);


CREATE TABLE "qtrack" (
        "rec_id" "int" IDENTITY NOT NULL PRIMARY KEY,
        "ip"     varchar(16)         NOT NULL,
        "qwords" varchar(4096)       NOT NULL,
        "qtime"  int                 NOT NULL,
	"wtime"  int                 NOT NULL,
        "nfound" int                 NOT NULL
);
CREATE INDEX qtrack_ipt ON qtrack(ip,qtime);


CREATE TABLE "qinfo" (
       "q_id"  int,
       "sname" text,
       "sval"  text
);
CREATE INDEX qinfo_id ON qinfo (q_id);


CREATE TABLE wrdstat (
  word varchar(64) NOT NULL,
  snd varchar(64) NOT NULL,
  cnt int NOT NULL
);
CREATE INDEX wrdstat_word ON wrdstat (word);
CREATE INDEX wrdstat_snd  ON wrdstat (snd);
